(*
 *
 *		AppleScript which handles input and opens Terminal
 *
 *		Modified from Marc Liyanage
 *			http://www.entropy.ch/software/applescript/
 *		and Jonathan Austin
 *			http://forums.macosxhints.com/showthread.php?p=426240#post426240 
 *		
 *)


-- ============
-- = Defaults =
-- ============

-- Make a new Terminal shell (0) or use the frontmost one 
property makeNew : 0
-- When creating Terminal shells, create new tabs (0) rather than new windows 
property useTabs : 0
-- Style to use for new tabs created by this script
property tabStyle : "Pro"


-- =================
-- = Even Handlers =
-- =================

-- `menu_click`, by Jacob Rus, September 2006
-- 
-- Accepts a list of form: `{"Finder", "View", "Arrange By", "Date"}`
-- Execute the specified menu item.  In this case, assuming the Finder 
-- is the active application, arranging the frontmost folder by date.

on menu_click(mList)
	local appName, topMenu, r
	
	-- Validate our input
	if mList's length < 3 then error "Menu list is not long enough"
	
	-- Set these variables for clarity and brevity later on
	set {appName, topMenu} to (items 1 through 2 of mList)
	set r to (items 3 through (mList's length) of mList)
	
	-- This overly-long line calls the menu_recurse function with
	-- two arguments: r, and a reference to the top-level menu
	tell application "System Events" to my menu_click_recurse(r, ((process appName)'s ¬
		(menu bar 1)'s (menu bar item topMenu)'s (menu topMenu)))
end menu_click

on menu_click_recurse(mList, parentObject)
	local f, r
	
	-- `f` = first item, `r` = rest of items
	set f to item 1 of mList
	if mList's length > 1 then set r to (items 2 through (mList's length) of mList)
	
	-- either actually click the menu item, or recurse again
	tell application "System Events"
		if mList's length is 1 then
			click parentObject's menu item f
		else
			my menu_click_recurse(r, (parentObject's (menu item f)'s (menu f)))
		end if
	end tell
end menu_click_recurse

on make_tab(style)
	my menu_click({"Terminal", "Shell", "New Tab", style})
end make_tab

on open untitled theObject
	-- This is triggered when the application icon is clicked
	
	set commandKeyDown to (call method "cmdKeyDown" of class "StartupController") as boolean
	if (commandKeyDown) then
		set useTabs to 0
	end if
	
	set optionKeyDown to (call method "optKeyDown" of class "StartupController") as boolean
	if (optionKeyDown) then
		set makeNew to 1
	end if
	
	-- Get the path of the frontmost Finder window
	tell application "Finder"
		try
			set thisFolder to (the target of the front window) as alias
		on error
			set thisFolder to startup disk
		end try
	end tell
	
	my openTerm(thisFolder)
	
	quit
	
end open untitled


on open theObject
	-- This is triggered when some documents/folders are dropped on the icon
	set commandKeyDown to (call method "cmdKeyDown" of class "StartupController") as boolean
	if (commandKeyDown) then
		set useTabs to 0
	end if
	
	set optionKeyDown to (call method "optKeyDown" of class "StartupController") as boolean
	if (optionKeyDown) then
		set makeNew to 1
	end if
	
	repeat with thisItem in theObject
		my openTerm(thisItem)
	end repeat
	
	quit
end open


-- =================
-- = Open Terminal =
-- =================
on openTerm(theItem)
	
	-- Get the POSIX path of the item (dropped or Finder selection)
	set thePath to POSIX path of theItem
	-- display dialog thePath
	
	-- Strip out file name to keep just directory (`dirname` like in Applescript)
	repeat until thePath ends with "/"
		set thePath to text 1 thru -2 of thePath
	end repeat
	-- display dialog thePath
	
	-- Set the command line to run
	set cdCommand to "cd " & quoted form of thePath
	
	-- `cd` to there in Terminal
	tell application "Terminal"
		activate
		
		if makeNew is equal to 0 then
			-- Always make a new shell
			
			if useTabs is equal to 0 then
				-- Make a new tab
				my make_tab(tabStyle)
				do script cdCommand in window 1
			else
				-- Make a new window
				do script cdCommand
			end if
			
			
		else
			-- Use the frontmost shell
			
			set windowCount to (count of the windows)
			if windowCount is greater than 0 then
				-- Check that there is at least a window
				
				if window 1 is not busy then
					-- Then check that that frontmost window is not busy
					-- If it is not busy, run the command there
					do script cdCommand in window 1
				else
					-- If it is busy run the command in a new window or tab depending on settings
					if useTabs is equal to 0 then
						-- Make a new tab
						my make_tab(tabStyle)
						do script cdCommand in window 1
					else
						-- Make a new window
						do script cdCommand
					end if
				end if
				
			else
				-- There's no window, so just create one
				do script cdCommand
				
			end if
			
		end if
		
	end tell
end openTerm
